Imports System.IO
Imports System.Xml.Serialization

Module modMain
    Private m_SettingsPath As String = "ImageScreenSaver.xml"
    Public imgSettings As New ImageSettings
    Public action As ActionType

    Public Enum ActionType
        actConfigure
        actPreview
        actRun
    End Enum
    
    Sub Main(ByVal args As String())
        Dim frmMain As New frmMain
        Dim frmOption As New frmOptions
    
        If VerifySettingsFile() Then
            LoadSettings()
        Else
            Application.Run(frmOption)
            Exit Sub
        End If

        'get our arguments
        If args.Length = 0 Then
            action = ActionType.actRun
        Else
            Select Case args(0).ToLower
                Case "/p"
                    action = ActionType.actPreview
                Case "/c"
                    action = ActionType.actConfigure
                Case "/s"
                    action = ActionType.actRun
            End Select
        End If

        'process our request
        Select Case action
            Case ActionType.actConfigure
                Application.Run(frmOption)
            Case ActionType.actPreview
                SetForm(frmMain, args(1))
                Application.Run(frmMain)
            Case ActionType.actRun
                Application.Run(frmMain)
        End Select
    End Sub

    Public Function VerifySettingsFile() As Boolean
        Dim f As File

        Return f.Exists(m_SettingsPath)
    End Function

    Private Sub SetForm(ByRef f As Form, ByRef arg As String)
        Dim style As Integer
        Dim previewHandle As Integer = Int32.Parse(CType(arg, String))
        Dim corners As New Win32Corners
        Dim r As New Win32Corners.RECT

        'get dimensions of preview
        corners.GetClientRect(previewHandle, r)

        With f
            .WindowState = FormWindowState.Normal
            .FormBorderStyle = FormBorderStyle.None
            .Width = r.right
            .Height = r.bottom
        End With

        'get and set new window style
        style = corners.GetWindowLong(f.Handle.ToInt32, corners.GWL_STYLE)
        style = style Or corners.WS_CHILD
        corners.SetWindowLong(f.Handle.ToInt32, corners.GWL_STYLE, style)

        'set parent window (preview window)
        corners.SetParent(f.Handle.ToInt32, previewHandle)

        'save preview in forms window structure
        corners.SetWindowLong(f.Handle.ToInt32, corners.GWL_HWNDPARENT, previewHandle)

        corners.SetWindowPos(f.Handle.ToInt32, 0, r.left, 0, r.right, r.bottom, _
            corners.SWP_NOACTIVATE Or corners.SWP_NOZORDER Or corners.SWP_SHOWWINDOW)
    End Sub

    Public Sub LoadSettings()
        Dim oRead As New StreamReader(m_SettingsPath)
        Dim x As New XmlSerializer(imgSettings.GetType)

        Try
            imgSettings = x.Deserialize(oRead)
        Catch ex As SystemException
        Finally
            oRead.Close()
        End Try
    End Sub

    Public Sub SaveSettings()
        Dim oWrite As New StreamWriter(m_SettingsPath)
        Dim x As New XmlSerializer(imgSettings.GetType)

        x.Serialize(oWrite, imgSettings)
        oWrite.Close()
    End Sub
End Module
